#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

import cgi
import cgitb; cgitb.enable()
import os
import os.path
from M2Crypto import X509, ASN1, m2, BIO, EVP
import time
import datetime
import tempfile
from types import *

import _xml
import lib.miarc_util


def existeCRL (idCA):
	"""Devuelve un lógico indicando si ya existe una CRL para la CA"""
	exist = False
	dirAct = os.getcwd()
	if os.path.exists(dirAct + '/CAs/' + idCA + '/' + idCA + 'PEM.crl'):
		exist = True
	return exist


def crearCRLVacia (idCA, callback, algFirma, dia=None, mes=None, anyo=None):
	""" Crea y configura una CRL vacía. 
	"""
	crlN = CRL()
	crlN.setLastUpdate()
	if dia != None and mes != None and anyo != None:	# se ha especificado fecha próxima actualización
		crlN.setNextUpdate(dia, mes, anyo)
	else:
		crlN.setNextUpdate()
	crlN.setIssuer(idCA)
	crlN.sign(idCA, callback, algFirma)
	crlN.writePem(idCA)


class CRL:
	def __init__(self, crlOriginal=None):
		""" Método constructor.
		"""
		if crlOriginal == None:
			self.crl = X509.CRL()
		else:
			self.crl = crlOriginal


	def setLastUpdate (self):
		""" Establece en la CRL la fecha de la última modificación como el momento 
		en que se llama al método.
		"""
		tCreacion = ASN1.ASN1_UTCTIME()
		ahora = int(time.time())
		tCreacion.set_time(ahora)
		self.crl.set_last_update(tCreacion)

	def setNextUpdate (self, dia=None, mes=None, anyo=None):
		""" Establece en la CRL la fecha de la próxima modificación la pasada como 
			argumento, si es 'None' será la fecha de última modificación.

			param type: int
		"""
		fechaAct = int(time.time())
		tActualizacion = ASN1.ASN1_UTCTIME()
		if dia != None and mes != None and anyo != None:
			# conseguir el valor en segundos desde el Epoch hasta la fecha de actualización
			fechaAct = datetime.datetime(anyo, mes, dia, 23, 0, 0)		# las 21:00 horas GMT
			fechaAct = int(time.mktime(fechaAct.timetuple()))
		tActualizacion.set_time(fechaAct)
		self.crl.set_next_update(tActualizacion)

	def setIssuer (self, idCA):
		""" Establece la entidad emisora de la CRL mediante el 'subject Name' de la CA.
		"""
		dirAct = os.getcwd()
		dirCACert = dirAct + '/CAs/' + idCA + '/cacert.pem'
		certCA = X509.load_cert(dirCACert)
		subjectName = certCA.get_subject()
		self.crl.set_issuer(subjectName)

	def addRevocado (self, certRev):
		""" Añade un certificado revocado a la CRL.
		"""
		self.crl.add_revoked(certRev)
		
	def sign (self, idCA, callback, algFirma):
		""" Firma la CRL.
		"""
		dirAct = os.getcwd()
		dirCAPrivKey = dirAct + '/CAs/' + idCA + '/private/cakey.pem'
		pkey = EVP.load_key(dirCAPrivKey, callback)		# creación del objeto EVP_PKEY
		self.crl.sign(pkey, algFirma)

	def writePem (self, idCA):
		""" Genera el archivo con extensión '.pem' que contiene a la CRL.
		"""
		dirAct = os.getcwd()
		bio = BIO.openfile (dirAct + '/CAs/' + idCA + '/' + idCA + 'PEM.crl', 'wb')
		m2.x509_crl_write_pem(bio.bio_ptr(), self.crl.crl)
		bio.close()
#		BIO.BIO.m2_bio_free(bio.bio_ptr())


# XXX ACTUALIZACION 14 AGOSTO XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX 
	def actualizar (self, idCA, callback, algFirma, dia=None, mes=None, anyo=None):
		""" Comprueba las fechas de caducidad de los certificados contenidos en 
			CRL, y si es anterior o igual a la actual los elimina de la CRL.

			Este proceso mantiene la CRL con el menor tamaño posible, facilitando 
			su distribución.
		"""
		# Este método debe ejecutarse cada vez que vaya a descargarse la CRL.

		cambioContenido = False
		dirAct = os.getcwd()
		dirCerts = dirAct + '/CAs/' + idCA + '/certs/'
		pilaRevocados = self.crl.get_revoked()					# obtenemos los certificados revocados de la CRL

		# contemplamos la posibilidad de que la pila esté vacía, es decir, la 
		# CRL no contenga certificados revocados
		if type(pilaRevocados) is not NoneType:
			pilaRevocadosAct = X509.Revoked_Stack()					# almacenaremos los que deben volver a la CRL
			tamPila = len (pilaRevocados)
			i = 0
			x = 0
			buf = BIO.MemoryBuffer()								# creamos el buffer de memoria
			while i<tamPila:
				certRev = pilaRevocados.pop()
				serialRev = certRev.get_serial_number()
				cert = X509.load_cert(dirCerts + str(serialRev) + ".pem")
				fechaCad = cert.get_not_after()
				m2.asn1_utctime_print(buf.bio_ptr(), fechaCad.asn1_utctime)
				fechaCadString = buf.getvalue()					# vacía el buffer
				fecLista = analisisCad (fechaCadString)
				# si la fecha de caducidad es posterior a la actual, se mete el certificado en la pila
				if lib.miarc_util.posFechaHoraAct (fecLista[0], fecLista[1], fecLista[2], fecLista[3], fecLista[4], fecLista[5]):
					pilaRevocadosAct.push(certRev)				# pilaRevocadosAct contendrá aquellos revocados que no hayan caducado
					x = x+1
				i = i+1


			crlAct = CRL()
			crlAct.setLastUpdate()
			crlAct.setNextUpdate(dia, mes, anyo)
			crlAct.setIssuer(idCA)
	
			crlActualizada = self.volcarPilaEnCRL(pilaRevocadosAct, crlAct)
			# una vez cargada la CRL se vuelve a firmar si se ha cambiado el contenido
			crlActualizada.sign(idCA, callback, algFirma)
			crlActualizada.writePem(idCA)
		else:													# la CRL estaba vacía
			# creamos otra CRL vacía firmada por el algoritmo pasado y con fecha 
			# próxima actualización la pasada creación de una CRL vacía
			crlActualizada = crearCRLVacia (idCA, callback, algFirma, dia, mes, anyo)
		return crlActualizada

# para empezar, get_revoked no vacía la crl
	def volcarPilaEnCRL (self, pila, crlAct):
		""" Descarga el contenido de la pila (X509_Rev) en la CRL.
		"""
		tam = len(pila)
		i = 0
		while i<tam:
			cert = pila.pop()
			crlAct.crl.add_revoked(cert)
			i = i+1
		return crlAct

# /XXX ACTUALIZACION 14 AGOSTO XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX 

def analisisCad (cadenaFecha):
	""" Recive una cadena con formato:  'Aug 10 17:01:43 2008 GMT' y devuelve 
		una lista de 6 enteros, de manera que:
		tupla(0) = dia
		tupla(1) = mes
		tupla(2) = anyo
		tupla(3) = hora
		tupla(4) = minuto
		tupla(5) = segundo
	"""
	listaCad = cadenaFecha.rsplit(' ')
	lista = ['', '', '', '', '', '']
	lista[0] = int(listaCad[1])		# dia
	lista[2] = int(listaCad[3])		# año
	mes = listaCad[0]				# mes
	if mes=='Jan':
		lista[1] = 1
	elif mes=='Feb':
		lista[1] = 2
	elif mes=='Mar':
		lista[1] = 3
	elif mes=='Apr':
		lista[1] = 4
	elif mes=='May':
		lista[1] = 5
	elif mes=='Jun':
		lista[1] = 6
	elif mes=='Jul':
		lista[1] = 7
	elif mes=='Aug':
		lista[1] = 8
	elif mes=='Sep':
		lista[1] = 9
	elif mes=='Oct':
		lista[1] = 10
	elif mes=='Nov':
		lista[1] = 11
	elif mes=='Dec':
		lista[1] = 12
	listaHora = listaCad[2].split(':')
	lista[3] = int(listaHora[0])
	lista[4] = int(listaHora[1])
	lista[5] = int(listaHora[2])
	return lista


class CertificadoRevocado:
	"Certificado digital X.509 Revocado"

	def __init__ (self):
		self.certRev = X509.Revoked()

	def setSerial (self, numSerie):
		self.certRev.set_serial_number(numSerie)			# numero de serie

	def setRevocationDate (self):
		tRevocacion = ASN1.ASN1_UTCTIME()
		tRevocacion.set_time(int(time.time()))
		self.certRev.set_revocation_date(tRevocacion)		# fecha de revocación

	def getSerial (self):
		numSerie = self.certRev.get_serial_number()
		return numSerie
	
	def getRevocationDate (self):
		""" Devuelve la fecha de revocación GMT en formato:
			'Mmm DD HH:MM:SS AAAA GMT'
		"""
		fechaRev = self.certRev.get_revocation_date()
		buf = BIO.MemoryBuffer()
		m2.asn1_utctime_print(buf.bio_ptr(), fechaRev.asn1_utctime)
		fechaRevString = buf.getvalue()
		return fechaRevString
