#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

import cgi
import cgitb; cgitb.enable()
import lib.miarc_util

class CFormulario:
	def __init__(self):
		""" Mapeo y control de errores en el servidor del formulario de creación 
			de una nueva CA importando certificado y claves RSA.

			@rtype: tupla.
			@return: objeto tupla.
			El primer elemento de la tupla indica si el formulario tiene 
			cumplimentados los campos obligatorios con valores válidos, 
				Si tupla[0]==False, tupla[1] es una cadena con un mensaje de error.
				Si tupla[0]==True, el resto de la tupla contiene los valores ingresados en los campos del formulario ordenados.
		"""

		self.error = []				# lista con los codigos de error detectados al parsear el contenido del formulario
		form = cgi.FieldStorage(keep_blank_values=1) 	# parsea la petición, form se comporta como un diccionario
		# "keep_blank_values=1" evita que FielStorage ignore aquellos campos que contengan cadenas vacias

		# Nombre Identificador de la CA	
		if form.has_key("idCA") and form["idCA"].value != "":
			if lib.miarc_util.contieneEspacios(form["idCA"].value):
				self.error.append(7)
				self.idCA = ''	# para que no salte una excepción al encontrarse en la línea 38 que self.idCA no tiene ningún valor asignado
			else:
				self.idCA = form["idCA"].value
		else:
			self.error.append(1) 		#"ERROR: Falta la información referente al Nombre Identificador de la CA."
			self.idCA = ''

		# Password
		if form.has_key("password") and form["password"].value != "":
			self.passw = form["password"].value
		else:
			self.error.append(2) #"ERROR: Introduzca un password para encriptar su clave RSA."

		# Verificación del password
		if form.has_key("vPassword") and form["vPassword"].value != "":
			vpassw = form["vPassword"].value
		else:
			self.error.append(3) #"ERROR: Rellene la casilla de verificación del password para encriptar su clave RSA."

		# Comprobando que coinciden las dos casillas de password
		if (form.has_key("password") and form["password"].value == "") or (form.has_key("vPassword") and form["vPassword"].value == "") or (self.passw != vpassw):
			self.error.append(4)		#"ERROR: Las casillas Password y Verificacion de Password NO coinciden."

		# Certificado
		if form.has_key("pathCert") and form["pathCert"].value != "":
			self.pathCert = form["pathCert"].value
		else:
			self.error.append(5) #ERROR: Falta el certificado

		# Clave Privada
		if form.has_key("pathKey") and form["pathKey"].value != "":
			self.pathKey = form["pathKey"].value
		else:
			self.error.append(6) #ERROR: Falta la clave privada


	def gestErrores (self):
		""" Devuelve una cadena enumerando los errores detectados en el formulario.
		"""
		msg = ""
		if len(self.error) != 0:
			msg += """<br/>Este mensaje de error ha sido generado por el servidor de la aplicación, esto implica que existe algún problema con la verificación de los datos en el cliente.<br/>

<br/>El problema puede deberse a varias causas, por favor compruebe que cumple los siguientes requerimientos:
	<blockquote><br/>1.- Asegúrese de utilizar un navegador web que permita ejecutar código javascript.
	<br/>2.- No trate de realizar peticiones al servidor sin usar los formularios suministrados y no altere su código fuente.
	<br/>3.- Compruebe que ha rellenado correctamente el formulario, principalmente los campos marcados con *.</blockquote>

<br/>A continuación se muestran los fallos detectados:<br/> 
"""
			for i in self.error:
				if i==1:
					msg += """<br/>ERROR (01NCAICK): Debe rellenar el campo: "*Nombre Identificador de la CA". \n"""
				if i==2:
					msg += """<br/>ERROR (02NCAICK): Debe rellenar el campo: "*Password". \n"""
				if i==3:
					msg += """<br/>ERROR (03NCAICK): Debe rellenar el campo: "*Verificación del Password". \n"""
				if i==4:
					msg += """<br/>ERROR (04NCAICK): Las casillas "*Password" y "*Verificación del Password" NO coinciden. \n"""
				if i==5:
					msg += """<br/>ERROR (05NCAICK): Debe seleccionar el archivo que contenga el Certificado Digital. \n"""
				if i==6:
					msg += """<br/>ERROR (06NCAICK): Debe seleccionar el archivo que contenga la Clave Privada RSA. \n"""
				if i==7:
					msg += """<br/>ERROR (07NCAICK): El campo "*Nombre Identificador de la CA" no admite espacios. \n"""
		return msg


	def vistaPrevia(self):
		""" Genera una página html con el contenido seleccionado para que el 
			usuario compruebe los valores elegidos.
			La página contiene campos <input type="hidden"> manteniendo un 
			formulario con la info suministrada oculta al usuario, permitiéndonos 
			parsearlo fácilmente mediante cgi.FieldStorage
		"""
		# generamos la vista previa html
		htmlCode = unicode ("""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>miARC - Información suministrada para configurar la nueva CA.</title>
	<link rel="stylesheet" type="text/css" href="../web/style.css" media="all">
	<script type="text/javascript" src="../web/view.js"></script>
	<script>
	function mostrar(enla) {
	  obj = document.getElementById('oculto');
	  obj.style.visibility = (obj.style.visibility == 'hidden') ? 'visible' : 'hidden';
	  enla.innerHTML = (enla.innerHTML == 'Ocultar Password') ? 'Mostrar Password' : 'Ocultar Password';
	}
	</script>
</head>
<body id="main_body" >
	<img id="top" src="../web/images/top.png" alt="">
	<img src="../web/images/miarc.jpg" alt="miARC" border="0" usemap="#myARCLogoMap" id="myARCLogo">
	<map name="myARCLogoMap" id="myARCLogoMap"><area shape="rect" coords="21,2,243,59" href="../web/index.html" /></map>
	<div id="form_container">
	<br>
	<form id="form_39443" class="appnitro" method="post" action="http://localhost:8000/cgi-bin/crearNuevaCAICK.py">
		<div class="form_description">
		<h2>Nueva Autoridad de Certificación.</h2>
		<p>A continuación se muestra la información que ha ingresado para crear la nueva autoridad de certificación, antes de continuar compruebe que todos los datos son correctos.</p>
		</div>
		<ul>
			<li id="li_1" >
				<label class="description" for="idCA">Nombre Identificador de la CA: </label>
				<div>
					<input type=hidden id="idCA" name="idCA" class="element text medium" type="text" maxlength="255" value='""" + self.idCA + """'/>
					""" + self.idCA + """
				</div>
			</li>
			<li id="li_2" >
				<label class="description" for="pathCert">Archivo conteniendo el Certificado Digital: </label>
				<div>
					<input type=hidden id="pathCert" name="pathCert" class="element text medium" type="text" maxlength="255" value='""" + self.pathCert + """'/>
					""" + self.pathCert + """
				</div>
			</li>
			<li id="li_3" >
				<label class="description" for="pathKey">Archivo conteniendo la Clave Privada RSA: </label>
				<div>
					<input type=hidden id="pathKey" name="pathKey" class="element text medium" type="text" maxlength="255" value='""" + self.pathKey + """'/>
					""" + self.pathKey + """
				</div>
			</li>
			<li class="section_break"></li>
			<li id="li_4" >
				<input type=hidden id="password" name="password" class="element text large" type="password" maxlength="255" value='""" + self.passw + """'/>
				<label class="description" for="password">Password: </label>
				Por razones de seguridad el password se mantiene oculto por defecto, si desea verlo pulse sobre el siguiente enlace: <a href="#" onclick="mostrar(this); return false" />Mostrar Password</a>.
				<div id="oculto" style="visibility:hidden">
					Password: """ + self.passw + """
				</div>
			</li>
			<li class="buttons">
				<input type="hidden" name="form_id" value="39443"/>
				<input id="backForm" class="button_text" type="button" name="submit" value="Cambiar" onclick="history.back()"/>
				<input id="saveForm" class="button_text" type="submit" name="submit" value="Continuar"/>
			</li>
		</ul>
	</form>
	</div>
	<img id="bottom" src="../web/images/bottom.png" alt="">
</body>
</html>""", 'utf-8')
		# Generación del código a imprimir
		print htmlCode.encode('utf-8')

if __name__ == '__main__':
	print "Content-Type: text/html"     # sigue HTML
	print								# linea en blanco, fin de la cabecera
	cf = CFormulario()
	cadErrores = cf.gestErrores()
	if len(cadErrores) == 0:
		cf.vistaPrevia()
	else:
		print cadErrores
