#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

import os
from xml.dom import minidom

class XML:
	def __init__(self, pathArcXml, nodoRaiz):
		""" Método constuctor. 

			param1: path absoluto del archivo XML que queremos crear
			param2: el nodo raíz del XML con formato: "<nodoRaiz>"
		"""
		self.nodoRaizIn = nodoRaiz
		self.nodoRaizFin = nodoRaiz[0] + '/' + nodoRaiz[1:]
		if not os.path.exists(pathArcXml):					# si no existe el archivo xml lo creamos
			etiquetaIni = """<?xml version="1.0" encoding="UTF-8"?>"""
			arcXml = open (pathArcXml, 'w')
			arcXml.write(etiquetaIni + '\n' + self.nodoRaizIn + '\n' + self.nodoRaizFin)
			arcXml.close()
		self.path = pathArcXml


class XMLRequest(XML):
	def newRequest (self, request, numReq, codigo):
		""" Crea una nueva entrada en el xml.

			param1: información recogida del formulario de requests
			param2: número identificador de la petición	
			param3: codigo de seguridad
		"""
		fXml = open (self.path, 'r+w')						# abre el xml ya existente 
		fXml.seek(-len(self.nodoRaizFin), 2)				# desplazamos a la penúltima linea
		lCert = self.generaEstXML(request, numReq, codigo)	# lista de cadenas con las filas de la estructura XML del certificado
		fXml.writelines(lCert)
		fXml.write('\n' + self.nodoRaizFin)
		fXml.close()


	def generaEstXML (self, request, numRequest, codigo):
		""" Recibe los datos captados del formulario de Nueva Petición y genera 
			una lista de cadenas que conforman una estructura XML 
			sintácticamente válida con la información a almacenar.

			param1: datos del formulario de nueva petición
			param2: id de la petición
		"""
		lXML = []

		numReq = "\t<request id='" + str(numRequest) + "'>"
		idDocument = "\n\t\t<idDocument>" + request.id + "</idDocument>"
		cnNodo = "\n\t\t<cn>" + request.cn + "</cn>"
		snNodo = "\n\t\t<sn>" + request.sn + "</sn>"
		gnNodo = "\n\t\t<gn>" + request.gn + "</gn>"
		oNodo = "\n\t\t<o>" + request.o + "</o>"
		ouNodo = "\n\t\t<ou>" + request.ou + "</ou>"
		emailNodo = "\n\t\t<email>" + request.mail + "</email>"
		serialNodo = "\n\t\t<serial>" + request.serial + "</serial>"
		locNodo = "\n\t\t<loc>" + request.loc + "</loc>"
		stNodo = "\n\t\t<st>" + request.st + "</st>"
		paisNodo = "\n\t\t<pais>" + request.pais + "</pais>"
		numBits = "\n\t\t<numBits>" + str(request.numBits) + "</numBits>"		
		exponente = "\n\t\t<exponente>" + request.exp + "</exponente>"
		password = "\n\t\t<password>" + request.passw + "</password>"
		algFirma = "\n\t\t<algFirma>" + request.firma + "</algFirma>"
		fechaCaducidad = "\n\t\t<fechaCad>" + request.frd + '-' + request.frm + '-' + request.fra + "</fechaCad>"
		usoNodo = "\n\t\t<uso>" + request.uso + "</uso>"
		codigoSec = "\n\t\t<codigo>" + codigo + "</codigo>"
		estado = "\n\t\t<estado>espera</estado>"			# inicialmente pendiente de ser tratada
		extNodo = "\n\t\t<ext>" + "PENDIENTE DE IMPLEMENTACION (EXTENSIONES)" + "</ext>"
		fin = "\n\t</request>"

		lXML.append(numReq)
		lXML.append(idDocument)
		lXML.append(cnNodo)
		lXML.append(snNodo)
		lXML.append(gnNodo)
		lXML.append(oNodo)
		lXML.append(ouNodo)
		lXML.append(emailNodo)
		lXML.append(serialNodo)
		lXML.append(locNodo)
		lXML.append(stNodo)
		lXML.append(paisNodo)
		lXML.append(numBits)
		lXML.append(exponente)
		lXML.append(password)
		lXML.append(algFirma)
		lXML.append(fechaCaducidad)
		lXML.append(usoNodo)
		lXML.append(codigoSec)
		lXML.append(estado)
		lXML.append(extNodo)
		lXML.append(fin)

		return lXML
