#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

import cgi
import cgitb; cgitb.enable()
import os
import lib.miarc_util

class CFormulario:
	def __init__(self):
		"""Mapeo y control de errores en el servidor del formulario de creación de una nueva petición de certificado digital.

		@rtype: tupla.
		@return: objeto tupla.
		El primer elemento de la tupla indica si el formulario tiene cumplimentados los campos obligatorios con valores validos, 
			Si tupla[0]==False, tupla[1] es una cadena con un mensaje de error.
			Si tupla[0]==True, el resto de la tupla contiene los valores ingresados en los campos del formulario ordenados.
		"""

		self.error = []									# lista con los codigos de error detectados al parsear el contenido del formulario
		form = cgi.FieldStorage(keep_blank_values=1)	# parsea la peticion, form se comporta como un diccionario
		# "keep_blank_values=1" evita que FielStorage ignore aquellos campos que contengan cadenas vacias

		# NIF/CIF (Identificación)
		if form.has_key("identificacion") and form["identificacion"].value != "":
			# TODO TRATAMIENTO PARA VER SI ES CORRECTO
			# Tiene que ser adaptado en la localización de la aplicación
			self.id = form["identificacion"].value
		else:
			self.error.append(1)		#ERROR

		# Nombre Comun (CN)
		if form.has_key("comName") and form["comName"].value != "":
			self.cn = form["comName"].value
		else:
			self.error.append(2)		#ERROR

		# Apellido (SN)
		if form.has_key("surname") and form["surname"].value != "":
			self.sn = form["surname"].value
		else:
			self.error.append(15)		#ERROR

		# Sobrenombre (GN)
		self.gn = form["givName"].value

		# Numero de Serie
		self.serial = form["serial"].value

		# Organizacion (O)
		if form.has_key("org") and form["org"].value != "":
			self.o = form["org"].value
		else:
			self.error.append(3)		#ERROR: valor obligatorio

		# Unidad Organizativa (OU)
		self.ou = form["uniOrg"].value

		# Email
		if form.has_key("emaCont") and form["emaCont"].value != "":
			self.mail = form["emaCont"].value
		else:
			self.error.append(4)		#ERROR: valor obligatorio

		# Localidad
		if form.has_key("ciudad") and form["ciudad"].value != "":
			self.loc = form["ciudad"].value
		else:
			self.error.append(5)   		#ERROR: valor obligatorio

		# Provincia
		if form.has_key("provincia") and form["provincia"].value != "":
			self.st = form["provincia"].value
		else:
			self.error.append(6)   		#ERROR: valor obligatorio

		# Pais
		if form.has_key("pais") and form["pais"].value != "":
			self.pais = form["pais"].value
		else:
			self.error.append(7)   		#ERROR: valor obligatorio

		# Comprobando que se ha marcado alguno de los usos del certificado
		if form.has_key("uso") and form["uso"].value != "":
			self.uso = form["uso"].value
		else:
			self.error.append(16)

		# Numero de Bits de la PrKey
		if form.has_key("numBits") and form["numBits"].value != "":
			opc = form["numBits"].value
			numBitsDic = {'0':'512', '1':'1024', '2':'2048', '3':'3072', '4':'4096'}
			self.numBits = numBitsDic[opc]
		else:
			self.numBits = '1024'			# valor por defecto

		# Exponente de Key
		if form.has_key("exponente") and form["exponente"].value != "":
			opcExp = form["exponente"].value
			expsDic = {'0':"RSA_3 (0x03L)", '1':"RSA_F4 (0x10001L)"}
			self.exp = expsDic[opcExp]
		else:
			self.exp = "RSA_F4 (0x10001L)"	# valor por defecto

		# Password
		if form.has_key("password") and form["password"].value != "":
			self.passw = form["password"].value
		else:
			self.error.append(8)		#"ERROR: Introduzca un password para encriptar su clave RSA."

		# Verificacion del password
		if form.has_key("vPassword") and form["vPassword"].value != "":
			vpassw = form["vPassword"].value
		else:
			self.error.append(9)		#"ERROR: Rellene la casilla de verificacion del password para encriptar su clave RSA."

		# Comprobando que coinciden las dos casillas de password
		if (form.has_key("password") and form["password"].value == "") or (form.has_key("vPassword") and form["vPassword"].value == "") or (self.passw != vpassw):
			self.error.append(10)		#"ERROR: Las casillas Password y Verificacion de Password NO coinciden."

		# Algoritmo de firma del certificado
		if form.has_key("firma") and form["firma"].value != "":
			self.firma = form["firma"].value
		else:
			self.firma = 'sha1'

		# Fecha de Caducidad
		# FR-dia
		if form.has_key("fechaCaduc_1") and form["fechaCaduc_1"].value != "":
			self.frd = form["fechaCaduc_1"].value
		else:
			self.error.append(11)		#"ERROR: Seleccione el dia en la fecha de revocacion del certificado."
		# FR-mes
		if form.has_key("fechaCaduc_2") and form["fechaCaduc_2"].value != "":
			self.frm = form["fechaCaduc_2"].value
		else:
			self.error.append(12)		#"ERROR: Seleccione el mes en la fecha de revocacion del certificado."
		#FR-anyo
		if form.has_key("fechaCaduc_3") and form["fechaCaduc_3"].value != "":
			self.fra = form["fechaCaduc_3"].value
		else:
			self.error.append(13)		#"ERROR: Seleccione el anyo en la fecha de revocacion del certificado."

		# Comprobando que la fecha de caducidad es posterior a la actual.
		if form.has_key("fechaCaduc_1") and form.has_key("fechaCaduc_2") and form.has_key("fechaCaduc_3") and form["fechaCaduc_1"].value != "" and form["fechaCaduc_2"].value != "" and form["fechaCaduc_3"].value != "":
			fechas = lib.miarc_util.posFechaAct (int(self.frd), int(self.frm), int(self.fra))
			if not fechas:
				self.error.append(14)		#"ERROR: La fecha introducida en el formulario no es posterior a la actual."


		# Número correspondiente al código de seguridad
		numCod = ''
		if form.has_key("numCod") and form["numCod"].value != "":
			numCod = form["numCod"].value
		else:
			self.error.append(17)

		# Comprobando código
		dirAct = os.getcwd()
		if numCod != '' and form.has_key("codigo") and form["codigo"].value != "":
			codigo = form["codigo"].value
			# comparandolo con el contenido del archivo "numCod"/codseg
			archCod = open(dirAct + "/web/images/codigos/" + numCod + "/codseg", 'r')
			if codigo != archCod.read(4):
				archCod.close()
				self.error.append(18)
		else:
			self.error.append(19)		#"ERROR: Introduzca el código de seguridad contenido en la imagen."

	def gestErrores (self):
		""" Devuelve una cadena enumerando los errores detectados en el formulario.
		"""
		msg = ""
		# Si el único error que se produce es el del código que no coincida con la imagen
		if len(self.error) == 1 and self.error.index(18) == 0:
			msg += """<br/>ERROR (18NC): El contenido del campo "*Código de Seguridad" no coincide con el de la imagen. \n"""

		# Si se produce otro/-s error/-es
		elif len(self.error) != 0:
			msg += """<br/>Este mensaje de error ha sido generado por el servidor de la aplicación, esto implica que existe algún problema con la verificación de los datos en el cliente.<br/>

<br/>El problema puede deberse a varias causas, por favor compruebe que cumple los siguientes requerimientos:
	<blockquote><br/>1.- Asegúrese de utilizar un navegador web que permita ejecutar codigo javascript.
	<br/>2.- No trate de realizar peticiones al servidor sin usar los formularios suministrados y no altere su código fuente.
	<br/>3.- Compruebe que ha rellenado correctamente el formulario, principalmente los campos marcados con *.</blockquote>

<br/>A continuación se muestran los fallos detectados:<br/> 
"""
			for i in self.error:
				if i==1:
					msg += """<br/>ERROR (01NC): Debe rellenar el campo: "*Número o Código de Identificación Fiscal". \n"""
				if i==2:
					msg += """<br/>ERROR (02NC): Debe rellenar el campo: "*Nombre Comun (CN)". \n"""
				if i==3:
					msg += """<br/>ERROR (03NC): Debe rellenar el campo: "*Organizacion (O)". \n"""
				if i==4:
					msg += """<br/>ERROR (04NC): Debe rellenar el campo: "*Email". \n"""
				if i==5:
					msg += """<br/>ERROR (05NC): Debe rellenar el campo: "*Datos de Localizacion -Ciudad". \n"""
				if i==6:
					msg += """<br/>ERROR (06NC): Debe rellenar el campo: "*Datos de Localizacion - Estado/Provincia/Region". \n"""
				if i==7:
					msg += """<br/>ERROR (07NC): Debe rellenar el campo: "*Datos de Localizacion - Pais". \n"""
				if i==8:
					msg += """<br/>ERROR (08NC): Debe rellenar el campo: "*Password". \n"""
				if i==9:
					msg += """<br/>ERROR (09NC): Debe rellenar el campo: "*Verificacion del Password". \n"""
				if i==10:
					msg += """<br/>ERROR (10NC): El contenido de los campos "*Password" y "*Verificacion del Password" NO coinciden. \n"""
				if i==11:
					msg += """<br/>ERROR (11NC): Debe rellenar el campo: "*Fecha de Caducidad - DD". \n"""
				if i==12:
					msg += """<br/>ERROR (12NC): Debe rellenar el campo: "*Fecha de Caducidad" - MM". \n"""
				if i==13:
					msg += """<br/>ERROR (13NC): Debe rellenar el campo: "*Fecha de Caducidad" - AAAA". \n"""
				if i==14:
					msg += """<br/>ERROR (14NC): La fecha introducida en el campo "*Fecha de Caducidad" debe ser posterior a la actual. \n"""
				if i==15:
					msg += """<br/>ERROR (15NC): Debe rellenar el campo: "*Apellido (SN)". \n"""
				if i==16:
					msg += """<br/>ERROR (16NC): Debe rellenar el campo: "*Uso del certificado". \n"""
				if i==17:
					msg += """<br/>ERROR (17NC): ¡¡¡No modifique el código del formulario!!!. \n"""
				if i==18:
					msg += """<br/>ERROR (18NC): El contenido del campo "*Código de Seguridad" no coincide con el de la imagen. \n"""
				if i==19:
					msg += """<br/>ERROR (19NC): Debe rellenar el campo "*Código de Seguridad". \n"""
		return msg


	def vistaPrevia(self):
		"""Esta funcion es llamada unicamente en el caso en que el formulario y su contenido sean correctos.
		"""
		# generamos la vista previa html
		htmlCodeIni = unicode ("""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>Nuevo Petición de Certificado Digital</title>
	<link rel="stylesheet" type="text/css" href="../web/view.css" media="all">
	<script type="text/javascript" src="../web/view.js"></script>
	<script type="text/javascript" src="../web/calendar.js"></script>
	<script>
	function mostrar(enla) {
	  obj = document.getElementById('oculto');
	  obj.style.visibility = (obj.style.visibility == 'hidden') ? 'visible' : 'hidden';
	  enla.innerHTML = (enla.innerHTML == 'Ocultar Password:') ? 'Mostrar Password' : 'Ocultar Password';
	}
	</script>
</head>
<body id="main_body" >
	<img id="top" src="../web/images/top.png" alt="">
	<img src="../web/images/miarc.jpg" alt="miARC" border="0" usemap="#myARCLogoMap" id="myARCLogo">
	<div id="form_container">
		<br>
		<form id="form_43076" class="appnitro"  method="post" action="http://localhost:8000/cgi-bin/newReq.py">
					<div class="form_description">
			<h2>Nueva Petición de Certificado Digital.</h2>
			<p>A continuación se encuentra la información que ha ingresado para crear una nueva petición de certificado digital, compruebe que todos los datos son correctos.</p>
		</div>						
		<ul>
		<li id="li_1" >
			<label class="description" for="comName">Número o Código de Identificación Fiscal: </label>
			<div>
				<input type=hidden id="identificacion" name="identificacion" class="element text medium" type="text" maxlength="255" value='""" + self.id + """'/>
				""" + self.id + """ 
			</div>
		</li>
		<li id="li_2" >
			<label class="description" for="comName">Nombre Común (CN): </label>
			<div>
				<input type=hidden id="comName" name="comName" class="element text medium" type="text" maxlength="255" value='""" + self.cn + """'/>
				""" + self.cn + """ 
			</div>
		</li>""", 'utf-8')

		snCode = unicode ("""<li id="li_3" >
			<label class="description" for="surname">Apellido (SN): </label>
			<div>
				<input type=hidden id="surname" name="surname" class="element text medium" type="text" maxlength="255" value='""" + self.sn + """'/> 
				""" + self.sn + """
			</div>
		</li>""", 'utf-8')

		gnCode = unicode ("""<li id="li_4" >
			<label class="description" for="givName">Sobrenombre (GN): </label>
			<div>
				<input type=hidden id="givName" name="givName" class="element text medium" type="text" maxlength="255" value="'""" + self.gn + """'"/> 
				""" + self.gn + """
			</div>
		</li>""", 'utf-8')

		serialCode = unicode ("""<li id="li_5" >
			<label class="description" for="serial">Número de Serie: </label>
			<div>
				<input type=hidden id="serial" name="serial" class="element text medium" type="text" maxlength="255" value='""" + self.serial + """'/> 
				""" + self.serial + """
			</div>
		</li>""", 'utf-8')

		orgCode = unicode ("""<li id="li_6" >
			<label class="description" for="org">Organización (O): </label>
			<div>
				<input type=hidden id="org" name="org" class="element text large" type="text" maxlength="255" value='""" + self.o + """'/> 
				""" + self.o + """
			</div>
		</li>""", 'utf-8')

		uniOrgCode = unicode ("""<li id="li_7" >
			<label class="description" for="uniOrg">Unidad Organizativa (OU): </label>
			<div>
				<input type=hidden id="uniOrg" name="uniOrg" class="element text large" type="text" maxlength="255" value='""" + self.ou + """'/> 
				""" + self.ou + """
			</div>
		</li>""", 'utf-8')

		emailCode = unicode ("""<li id="li_8" >
			<label class="description" for="emaCont">Email: </label>
			<div>
				<input type=hidden id="emaCont" name="emaCont" class="element text large" type="text" maxlength="255" value='""" + self.mail + """'/> 
				""" + self.mail + """
			</div>
		</li>""", 'utf-8')

		dirCode = unicode ("""<li id="li_9" >
				<label class="description" for="direccion">Datos de Localización: </label>
			</li>
			<li id="li_10" >
				<label class="description" for="ciudad">Localidad: </label>
				<div>
					<input type=hidden id="ciudad" name="ciudad" class="element text medium" value='""" + self.loc + """' type="text"/>
					""" + self.loc + """
				</div>
			</li>
			<li id="li_10" >
				<label class="description" for="provincia">Estado / Provincia / Región: </label>
				<div>
					<input type=hidden id="provincia" name="provincia" class="element text medium" value='""" + self.st + """' type="text"/>
					""" + self.st + """
				</div>
			</li>
			<li id="li_10" >
				<label class="description" for="pais">País: </label>
				<div>
					<input type=hidden id="pais" name="pais" class="element text medium" value='""" + self.pais + """' type="text"/>
					""" + self.pais + """
				</div>
			</li>""", 'utf-8')

		# Tratando los posibles usos del certificado
		if self.uso == '1':
			extSet = "SSL Client"
		elif self.uso == '2':
			extSet = "SSL Client CA"
		elif self.uso == '3':
			extSet = "SSL Server"
		elif self.uso == '4':
			extSet = "SSL Server CA"
		elif self.uso == '5':
			extSet = "Netscape SSL Server"
		elif self.uso == '6':
			extSet = "S/MIME Signing"
		elif self.uso == '7':
			extSet = "S/MIME Encryption"
		elif self.uso == '8':
			extSet = "S/MIME CA"
		elif self.uso == '9':
			extSet = "CRL Signing"
		elif self.uso == '10':
			extSet = "CRL Signing CA"

		usosCode = unicode ("""<li id="li_20" >
			<label class="description" for="uso">Uso del certificado: </label>
			<div>
				<input type=hidden id="uso" name="uso" class="element text medium" type="text" maxlength="255" value='""" + self.uso + """'/>
				""" + extSet + """
			</div>
		</li>""", 'utf-8')

		posCode = unicode ("""<li class="section_break"></li>
		<li id="li_17" >
			<label class="description" for="numBits">Número de bits de la Clave Privada: </label>
			<div>
				<input type=hidden id="numBits" name="numBits" class="element text small" type="text" maxlength="255" value='""" + self.numBits + """'/> 
				""" + self.numBits + """
			</div> 
		</li>
		<li id="li_18" >
			<label class="description" for="exp">Exponente: </label>
			<div>
				<input type=hidden id="exp" name="exp" class="element text small" type="text" maxlength="255" value='""" + self.exp + """'/> 
				""" + self.exp + """
			</div>
		</li>
		<li id="li_12" >
			<input type=hidden id="password" name="password" class="element text large" type="password" maxlength="255" value='""" + self.passw + """'/>
			<label class="description" for="password">Password: </label>
			Por razones de seguridad el password se mantiene oculto por defecto, si desea verlo pulse sobre el siguiente enlace: <a href="#" onclick="mostrar(this); return false" />Mostrar Password</a>.
			<div id="oculto" style="visibility:hidden">
				Password: """ + self.passw + """
			</div>
		</li>
		<li id="li_19" >
			<label class="description" for="firma">Algoritmo de Firma: </label>
			<div>
				<input type=hidden id="firma" name="firma" class="element text large" type="firma" maxlength="255" value='""" + self.firma + """'/>
				""" + self.firma + """
			</div>
		</li>
		<li class="section_break"></li>		
		<li id="li_15" >
			<label class="description" for="fechaCaduc">*Fecha de Caducidad: </label>
			<span>
				<input type=hidden id="fechaCaduc_1" name="fechaCaduc_1" class="element text" size="2" maxlength="2" value='""" + self.frd + """' type="text"/>
			</span>
			<span>
				<input type=hidden id="fechaCaduc_2" name="fechaCaduc_2" class="element text" size="2" maxlength="2" value='""" + self.frm + """' type="text"/>
			</span>
			<span>
		 		<input type=hidden id="fechaCaduc_3" name="fechaCaduc_3" class="element text" size="4" maxlength="4" value='""" + self.fra + """' type="text"/>
			</span>
			""" + self.frd + '/' + self.frm + '/' + self.fra + """
		</li>		
		<li class="buttons">
		    <input type="hidden" name="form_id" value="43076" />
		    <input id="backForm" class="button_text" type="button" name="submit" value="Cambiar" onclick="history.back()"/>
			<input id="saveForm" class="button_text" type="submit" name="submit" value="Continuar"/>
		</li>
		</ul>
		</form>	
		<div id="footer">
			<p align="center">Creado por Francisco José González Troyano para la Asignatura Proyecto Informático de la <a href="http://www.informatica.us.es">ETSII</a></p>
		</div>
	</div>
	<img id="bottom" src="../web/images/bottom.png" alt="">
</body>
</html>""", 'utf-8')
		# Generacion de codigo a imprimir
		html = htmlCodeIni
		if self.sn != '':
			html += snCode
		if self.gn != '':
			html += gnCode
		if self.serial != '':
			html += serialCode
		html += orgCode
		if self.ou != '':
			html += uniOrgCode
		html += emailCode
		html += dirCode
		html += usosCode
		html += posCode
		print html.encode('utf-8')


if __name__ == '__main__':
	print "Content-Type: text/html"     # sigue HTML
	print								# linea en blanco, fin de la cabecera
	cf = CFormulario()
	cadErrores = cf.gestErrores()
	if len(cadErrores) == 0:
		cf.vistaPrevia()
	else:
		print cadErrores
